/*******************************************************************************************************
    Search for classical Capacity of Quantum Channels(SeCQC) Version 1.0, a program that can calculate the maximum accessible information 
    and the corresponding optimal statistical operators as well as POVMs given a certain quantum channel.

    Copyright (C) 2010 J.W. Shang, K.L. Lee and B.-G. Englert.

    This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License Version 3 as published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. 
    See the GNU General Public License for more details.

	You can refer to http://www.gnu.org/licenses/ for the GNU General Public License.
	
	Detail manual for the program could be downloaded from http://www.quantumlah.org/publications/software/SeCQC/.
    
    The code for Graphical User Interface (GUI) is following the book 'Cross-Platform GUI Programming with WxWidgets'.
********************************************************************************************************/
//******************************************************************************************************/
//	Compiled under Windows XP Service Pack 3, with wxWidgets 2.8.10, MinGW 5.1.6, MSYS 1.0.11
//
//	Compile with the following command
//		g++ MI.cpp `wx-config --libs` `wx-config --cxxflags` -o SeCQC
//	in MSYS shell.
//	
//  If you face problems running the program in a Linux environment, try
//      export LD LIBRARY PATH=/usr/local/lib
//  in command window.
//
//******************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string> /*for string variables */
#include <vector> /* for manipulating variable matrix during runtime */
#include <complex> /* for complex double, creal, cimag, csqrt, complex arithmetic */
#include <cmath> /* for log */
#include <ctime> /* for time */

//--------------------------------------------------------------
//	import headers for GUI
//--------------------------------------------------------------
#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/stattext.h>
#include <wx/statbox.h>
#include <wx/spinctrl.h>
#include <wx/grid.h>
#include <wx/radiobox.h>
#include <wx/filedlg.h>
#include <wx/textfile.h>
#include <wx/tokenzr.h>
//--------------------------------------------------------------
//	end of import of headers for GUI
//--------------------------------------------------------------

using namespace std;
//define complex double data type
typedef complex<double> dcomp;

#include "functionMI.h" /* class header files for functions calculating mutual information */
#include "MI.h" /* class header files for GUI */

IMPLEMENT_APP(myApp);

//----------------------------------------------------------
//	functions that do not belong to any class
//----------------------------------------------------------
//convert a string into a double
double convertString(wxString myString){
	double myDouble;
	if(myString.ToDouble(&myDouble)){
		return myDouble;
	};	
}
	
//----------------------------------------------------------
//	Event handlers for myAPP
//----------------------------------------------------------
bool myApp::OnInit()
{
	myFrame *myframe = new myFrame(_("SeCQC Version 1.0"), wxDefaultPosition, wxSize(300,100));
	myframe->Show(true);
	SetTopWindow(myframe);
	
	return TRUE;
}

//----------------------------------------------------------
//	matrixGrid
//----------------------------------------------------------
matrixGrid::matrixGrid(wxPanel *panel, int wxID)
	   : wxGrid(panel,wxID,wxDefaultPosition,wxSize(500,85))
{	
	CreateGrid(4,4);
	SetRowLabelSize(0);
	SetColLabelSize(0);
	SetDefaultColSize(60,true);
	SetDefaultRowSize(17,true);
	
	//set to display only floating point numbers
	for(int i=0;i<3;i++)
		SetColFormatFloat(i,-1,4);
}

//modify the number of columns and rows displayed
void matrixGrid::setDimension(int x)
{
	int j;
	int i=GetNumberCols();
	if(x>i){
		//need to increase the number of columns and the number of rows
		InsertRows( i, x-i, true );
		InsertCols( i, x-i, true );
	}else if(x<i){
		//need to decrease the number of columns and the number of rows
		DeleteRows( 0, i-x, true );
		DeleteCols( 0, i-x, true );
	}else{
		//do nothing since the rho has the same dimension
	}	
	//set to display only floating point numbers	
	for(i=0;i<x;i++)
		SetColFormatFloat(i,-1,4);
}

//modify the value at the (i,j)th cell to the value f
void matrixGrid::changeElement(int i,int j,double f)
{
	wxString s = "";
	s += wxString::Format(_T("%f"),f);
	
	SetCellValue(i,j,s);
}

//display the matrix a of dimension m X n
void matrixGrid::setGrid(vector< vector<dcomp> > a, int m, int n, int ri)
{
	int i,j;
	wxString s;
	for(i=0;i<m;i++){
		for(j=0;j<n;j++){			
			s = "";
			if(ri == 0) s += wxString::Format(_T("%f"),real(a[i][j])); //real part
			if(ri == 1) s += wxString::Format(_T("%f"),imag(a[i][j])); //imaginary part
			SetCellValue(i,j,s);
		}
	}
}

//----------------------------------------------------------
//	myPanel
//----------------------------------------------------------

//----------------------------------------------------------
//	Implementation of GUI
//----------------------------------------------------------
myPanel::myPanel( wxFrame *frame, int x, int y)
       : wxPanel( frame, wxID_ANY, wxPoint(x, y))
{
    //initialisation of variables
	my_panel = NULL;
	m_book = NULL;
	m_calcButton = NULL;
	m_JTEXT = NULL;
	m_KTEXT = NULL;
	m_CapDimension = NULL;
	m_CapDimensionText = NULL;
	m_rhoDimension = NULL;
	m_rhoDimensionText = NULL;
	m_MTEXT = NULL;
	m_CTEXT = NULL;
	m_directGradChance = NULL;
	m_InputCapC = NULL;
	m_InputCapM = NULL;
	CapGrid = NULL;
	rhoGrid = NULL;
	povmGrid = NULL;
	m_CapRadio = NULL;
	m_rhoRadio = NULL;
	m_povmRadio = NULL;
	m_Displayrho = NULL;
	m_DisplayPOVM = NULL;
	m_displayMI = NULL;
	m_tol_MI = NULL;
	m_output = NULL;
	m_directory = NULL;
	m_directChange = NULL;
	
    //to display "Real" or "Imaginary" radio button
	wxArrayString strings;
	strings.Add(wxT("&Real"));
	strings.Add(wxT("&Imaginary"));
	
	//Create the notebook with tabs
	m_book = new wxNotebook(this, BOOKCTRL);
	
 	my_panel = new wxPanel(m_book);
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL); 	
	 	 	
 	wxFlexGridSizer *topsizer = new wxFlexGridSizer(4,0,0);
 	topsizer->AddGrowableCol(1);
 	
 	//-------------------------------------------------------------------
 	//	sizer for first static box
 	//-------------------------------------------------------------------
 	//add the first paramenter input: number of rho_j
 	wxStaticText *text = new wxStaticText(my_panel, wxID_ANY, _T("No. of Quantum States J ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_JTEXT = new wxSpinCtrl(my_panel, wxID_JTEXT, _T("2"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,100,2);
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(m_JTEXT,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
	
	//add the second paramenter input: number of pi_k
 	text = new wxStaticText(my_panel, wxID_ANY, _T("No. of POVM outcomes K ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_KTEXT = new wxSpinCtrl(my_panel, wxID_KTEXT, _T("5"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,100,3); 	
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(m_KTEXT,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
	
	//add the third paramenter input: number of Channels
 	text = new wxStaticText(my_panel, wxID_ANY, _T("No. of Quantum Channels C ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_CTEXT = new wxSpinCtrl(my_panel, wxID_CTEXT, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,100,3); 	
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(m_CTEXT,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
			
	//add the fourth paramenter input: number of Kraus operators Cap_m
 	text = new wxStaticText(my_panel, wxID_ANY, _T("No. of Kraus Operators M ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_MTEXT = new wxSpinCtrl(my_panel, wxID_MTEXT, _T("2"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,100,2); 	
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(m_MTEXT,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
	
	//add the fifth parameter input: dimension of rho_j
	text = new wxStaticText(my_panel, wxID_ANY, _T("Dimension of  N ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	wxBoxSizer *temp_sizer1 = new wxBoxSizer(wxHORIZONTAL);
    m_rhoDimension = new wxSpinCtrl(my_panel,wxID_rhoDimension, _T("2"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 80, 3);
 	m_rhoDimensionText = new wxStaticText(my_panel,wxID_rhoDimensionText, "2", wxDefaultPosition, wxDefaultSize);
 	temp_sizer1->Add(m_rhoDimension, 0, wxGROW|wxALL,0);
 	temp_sizer1->Add(new wxStaticText(my_panel,wxID_ANY, _T("X"), wxDefaultPosition, wxDefaultSize), 0, wxALL,5);
 	temp_sizer1->Add(m_rhoDimensionText, 0, wxALL,5);
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(temp_sizer1,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
	
	//add the sixth parameter input: dimension of Cap_m
	text = new wxStaticText(my_panel, wxID_ANY, _T("Dimension of Cap D ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	wxBoxSizer *temp_sizer = new wxBoxSizer(wxHORIZONTAL);
 	m_CapDimension = new wxSpinCtrl(my_panel,wxID_CapDimension, _T("2"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1, 80, 3);
 	m_CapDimensionText = new wxStaticText(my_panel,wxID_CapDimensionText, "2", wxDefaultPosition, wxDefaultSize);
 	temp_sizer->Add(m_CapDimension, 0, wxGROW|wxALL,0);
 	temp_sizer->Add(new wxStaticText(my_panel,wxID_ANY, _T("X"), wxDefaultPosition, wxDefaultSize), 0, wxALL,5);
 	temp_sizer->Add(m_CapDimensionText, 0, wxALL,5);
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(temp_sizer,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
		
	//add the seventh parameter input: percentage chance to use direct gradient in the steepest ascent method
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Percentage chance using DG ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_directGradChance = new wxSpinCtrl(my_panel, wxID_ANY, _T("0"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 0,100,0);
 	topsizer->Add(text,0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	topsizer->Add(m_directGradChance,0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);	
						  
 	//add the eighth parameter input: tolerance in MI
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Tolerance in MI ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_tol_MI = new wxTextCtrl(my_panel, wxID_ANY, _T("0.000001"),wxDefaultPosition, wxDefaultSize);
 	topsizer->Add(text,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
 	topsizer->Add(m_tol_MI,0,wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);			
 	 	
 	//add the ninth parameter input: the output file
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Output file ="), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_output = new wxTextCtrl(my_panel, wxID_ANY, _T("output.txt"),wxDefaultPosition, wxDefaultSize);
 	topsizer->Add(text,0,wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
 	topsizer->Add(m_output,0,wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL,5);
 	
 	// add the tenth parameter input: output file directory
 	text = new wxStaticText(my_panel, wxID_ANY, _T("Save output file to"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT);
 	m_directory = new wxTextCtrl(my_panel, wxID_ANY, _T("C:\\"), wxDefaultPosition, wxDefaultSize);
 	m_directChange = new wxButton(my_panel, BUTTON_Save, _T("&..."), wxDefaultPosition, wxDefaultSize);
 	wxBoxSizer *temp_sizer3 = new wxBoxSizer(wxHORIZONTAL);
 	temp_sizer3->Add(m_directory, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxGROW|wxALL, 5);
 	temp_sizer3->Add(m_directChange, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 	topsizer->Add(text, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

	topsizer->Add(temp_sizer3, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 0);
 	 	
 	wxBoxSizer *staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Parameter Settings")), wxVERTICAL );
 	staticbox->Add(topsizer,wxSizerFlags().Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags().Expand().Border(wxALL, 0));
 	
 	//-------------------------------------------------------------------
 	//	sizer for second static box
 	//------------------------------------------------------------------- 	
	//add parameter to display the input Kruas Operators Cap_m
	wxBoxSizer* topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	m_InputCapC = new wxSpinCtrl(my_panel, wxID_InputCapC, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,1,1);
	m_InputCapM = new wxSpinCtrl(my_panel, wxID_InputCapM, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,2,1);
	m_CapRadio = new wxRadioBox(my_panel, wxID_ANY, wxT("Display Real or Imaginary"), wxDefaultPosition, wxDefaultSize, strings, 2, wxRA_SPECIFY_COLS);	
	temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Matrix representation of Cap")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_InputCapC, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_InputCapM, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_CapRadio, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);	
	CapGrid = new matrixGrid(my_panel,wxID_CapGrid);
		 	 																		    
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
	topsizer2->Add(CapGrid, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Input Matrix Representation of Kraus Operators")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));
 	
 	//-------------------------------------------------------------------
 	//	sizer for third static box
 	//------------------------------------------------------------------- 	
	//add parameter to display the optimized quantum states rho
	topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	m_Displayrho = new wxSpinCtrl(my_panel, wxID_Displayrho, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,2,1);
	m_rhoRadio = new wxRadioBox(my_panel, wxID_ANY, wxT("Display Real or Imaginary"), wxDefaultPosition, wxDefaultSize, strings, 2, wxRA_SPECIFY_COLS);	
	temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Display optimal density operator ")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_Displayrho, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_rhoRadio, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);	
	rhoGrid = new matrixGrid(my_panel,wxID_rhoGrid);
	rhoGrid->EnableEditing(true);
		 	 																		    
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
	topsizer2->Add(rhoGrid, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Display Optimal Quantum States")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));	

 	//-------------------------------------------------------------------
 	//	sizer for fourth static box
 	//------------------------------------------------------------------- 	
	//add parameter to display optimal POVM
	topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	m_DisplayPOVM = new wxSpinCtrl(my_panel, wxID_DisplayPOVM, _T("1"), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS, 1,5,1);
	m_povmRadio = new wxRadioBox(my_panel, wxID_ANY, wxT("Display Real or Imaginary"), wxDefaultPosition, wxDefaultSize, strings, 2, wxRA_SPECIFY_COLS);
	temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Display the optimal ")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);	
	temp_sizer->Add(m_DisplayPOVM, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer->Add(m_povmRadio, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	povmGrid = new matrixGrid(my_panel,wxID_povmGrid);
	povmGrid->EnableEditing(true);
	
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
	topsizer2->Add(povmGrid, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Display Optimal POVM outcomes")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));	
 	
 	//-------------------------------------------------------------------
 	//	sizer for fifth static box
 	//------------------------------------------------------------------- 	
	//add parameter to display maximal mutual information and control button
	topsizer2 = new wxBoxSizer(wxVERTICAL);
	temp_sizer = new wxBoxSizer(wxHORIZONTAL);
	temp_sizer->Add(new wxStaticText(my_panel, wxID_ANY, _T("Maximal Mutual Information I =")), 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	m_displayMI = new wxTextCtrl(my_panel, wxID_DisplayMI, wxEmptyString, wxDefaultPosition, wxDefaultSize);
	temp_sizer->Add(m_displayMI, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	wxBoxSizer *temp_sizer2 = new wxBoxSizer(wxHORIZONTAL);
	m_calcButton = new wxButton(my_panel, BUTTON_Calc,_T("&Calculate MI"), wxDefaultPosition, wxSize(100,30));
	temp_sizer2->Add(m_calcButton, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	temp_sizer2->Add(new wxButton(my_panel, BUTTON_Reset,_T("&Reset"), wxDefaultPosition, wxSize(100,30)), 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL,5);
	
	topsizer2->Add(temp_sizer, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
	topsizer2->Add(temp_sizer2, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL,0);
 	staticbox = new wxStaticBoxSizer(new wxStaticBox(my_panel, wxID_ANY, _T("Display the Maximal Mutual Information")), wxVERTICAL );
 	staticbox->Add(topsizer2,wxSizerFlags(1).Expand().Border(wxALL, 5));
 	sizer->Add(staticbox,wxSizerFlags(1).Expand().Border(wxALL, 0));	 	
 	
 	my_panel->SetSizer(sizer);
 	m_book->AddPage(my_panel, _T("Untitled"), true);

 	//Place everything in a sizer and attach to the frame
 	sizer = new wxBoxSizer(wxVERTICAL);
 	sizer->Add(m_book,wxSizerFlags(1).Expand().Border(0));
 	SetSizer(sizer);
 	sizer->SetSizeHints(frame);
 	
 	m_information = new MutInf(m_CapDimension->GetValue(),m_rhoDimension->GetValue(),m_JTEXT->GetValue(),m_KTEXT->GetValue(),m_MTEXT->GetValue(),m_CTEXT->GetValue());
 	CapGrid->setGrid(m_information->getCap(0, 0),m_CapDimension->GetValue(),m_CapDimension->GetValue(),0);
}

BEGIN_EVENT_TABLE(myPanel,wxPanel)
	EVT_SPINCTRL(wxID_JTEXT, myPanel::ChangerhoNumber)
	EVT_SPINCTRL(wxID_KTEXT, myPanel::ChangePOVMNumber)
	EVT_SPINCTRL(wxID_CTEXT, myPanel::ChangeCapNumber)
	EVT_SPINCTRL(wxID_MTEXT, myPanel::ChangeCapNumber)
	EVT_SPINCTRL(wxID_CapDimension, myPanel::ChangeCapDimension)
	EVT_SPINCTRL(wxID_rhoDimension, myPanel::ChangerhoDimension)
	EVT_SPINCTRL(wxID_rhoDimension, myPanel::ChangePOVMDimension)
	EVT_SPINCTRL(wxID_InputCapC,myPanel::UpdateCap)
	EVT_SPINCTRL(wxID_InputCapM,myPanel::UpdateCap)
	EVT_SPINCTRL(wxID_Displayrho,myPanel::Updaterho)
	EVT_SPINCTRL(wxID_DisplayPOVM,myPanel::UpdatePOVM)
	EVT_MENU(wx_Calc,myPanel::OnCalc)
	EVT_GRID_CMD_CELL_CHANGE(wxID_rhoGrid, myPanel::OnrhoCellValueChanged)
	EVT_GRID_CMD_CELL_CHANGE(wxID_povmGrid, myPanel::OnPOVMCellValueChanged)
    EVT_GRID_CMD_CELL_CHANGE(wxID_CapGrid, myPanel::OnCapCellValueChanged)
	EVT_BUTTON(BUTTON_Calc, myPanel::OnCalc)
	EVT_BUTTON(BUTTON_Reset, myPanel::OnReset)
	EVT_BUTTON(BUTTON_Save, myPanel::OnSave)
END_EVENT_TABLE()

//----------------------------------------------------------------------
//	myFrame
//----------------------------------------------------------------------
#include "sample.xpm"
myFrame::myFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
		:wxFrame((wxFrame *)NULL,-1,title,pos,size)
{
	SetIcon(wxIcon(sample_xpm));
	wxMenu *myFilemenu = new wxMenu;
	myFilemenu->Append(wxID_IMPORT,"&Import\tCtrl+I","Import Channels");
	myFilemenu->Append(wxID_EXIT,"&Exit\tAlt+F4","Quit");
	
	wxMenu *myCalcmenu = new wxMenu;
	myCalcmenu->Append(wx_Calc,"&Calculate\tCtrl+C","Calculate Mutual Information");
	
	wxMenu *myAboutmenu = new wxMenu;
	myAboutmenu->Append(wxID_ABOUT,"&About","About SeCQC");
	
	wxMenuBar *mymenubar = new wxMenuBar();
	mymenubar->Append(myFilemenu,"&File");
	mymenubar->Append(myCalcmenu,"&Calc");
	mymenubar->Append(myAboutmenu,"&About");
	
	SetMenuBar(mymenubar);
	
	//Create status bar with 3 fields for the window
	CreateStatusBar(2);
	SetStatusText(_T("Ready"),0);

	//myPanel *panel = new myPanel(this, 0, 0);
	m_panel = new myPanel(this, 0, 0);
}

BEGIN_EVENT_TABLE(myFrame,wxFrame)
	EVT_MENU(wxID_IMPORT,myFrame::OnImport)
	EVT_MENU(wxID_EXIT,myFrame::OnQuit)
	EVT_MENU(wxID_ABOUT,myFrame::OnAbout)
END_EVENT_TABLE()

//-----------------------------------------------------------
//	Event Handlers for myFrame
//-----------------------------------------------------------
//to import the value of D,N,J,K,M,C from a file
int myFrame::extractParameter(wxString& text)
{
	int equalIndex = text.find("=");
	size_t stringLength = text.Len();
	wxString value = text.Mid(equalIndex+1, stringLength - equalIndex-1);
	value.Trim(true);
	value.Trim(false);
	
	int i;
	long l;
	if(value.ToLong(&l,10)){
		i = l;
		return i;
	}
}

//to import the matrix elements of each Cap_m
void myFrame::extractCapElement(wxString& text, int l, int m, int& dimen_Cap)
{
	int i,k;
	int imindex, imsignindex, plusindex, minusindex;
	double reNo, imNo;
	wxString tempString,s;
	wxStringTokenizer dummyString;
	int equalIndex = text.find("=");
	size_t stringLength = text.Len();
	wxString value = text.Mid(equalIndex+1, stringLength - equalIndex-1);
	value.Trim(true);
	value.Trim(false);
	
	vector<wxString> row;
	wxStringTokenizer tkz(value, wxT("}")); //split the imported line into rows of matrix with "}" as the delimiter
	while (tkz.HasMoreTokens())
	{
    	wxString token = tkz.GetNextToken();
		token = token.AfterLast('{');
    	row.push_back(token);
	}
        m_panel->m_InputCapC->SetValue(l);
	m_panel->m_InputCapM->SetValue(m);
	for(i=0;i<dimen_Cap;i++){
		k=0;
		dummyString.SetString(row[i],",");
		while(dummyString.HasMoreTokens())
		{
			tempString = dummyString.GetNextToken();
			tempString.Trim(true);
			tempString.Trim(false);
			imindex = tempString.find("I");
			if(imindex != wxNOT_FOUND){ //there is an imaginary part of the number	
				//look for the starting position of the imaginary part
				plusindex = tempString.find("+",true); //look for + sign from the back
				minusindex = tempString.find("-", true); //look for - sign from the back
				if(plusindex == wxNOT_FOUND) plusindex = 0;
				if(minusindex == wxNOT_FOUND) minusindex = 0;
				imsignindex = (plusindex>minusindex || plusindex==minusindex)? plusindex:minusindex;

				value = tempString.Mid(imsignindex, imindex-imsignindex);
				value.Trim(true);
				value.Trim(false);
				imNo = convertString(value);
				if(imsignindex == 0){
					reNo =0;  //no real part
				}else{
					value = tempString.Mid(0,imsignindex);
					value.Trim(true);
					value.Trim(false);
					reNo = convertString(value);
				}
			}else{ //there is only real part
				reNo = convertString(tempString);
				imNo = 0;
			}
			
			if(m_panel->m_CapRadio->GetSelection() == 0){ //display real part
				s = "";
				s += wxString::Format(_T("%f"),reNo);
				m_panel->CapGrid->SetCellValue(i,k,s);
			}else{ //display imaginary part
				s = "";
				s += wxString::Format(_T("%f"),imNo);
				m_panel->CapGrid->SetCellValue(i,k,s);
			}
			
			m_panel->m_information->setCapElement(l-1,m-1,i,k,reNo,0);
			m_panel->m_information->setCapElement(l-1, m-1,i,k,imNo,1);
			k++;
		}
	}
}

void myFrame::OnImport(wxCommandEvent& e)
{
	//open a file dialog to import file
	wxString caption = _T("Choose a text file to import the Channels");
	wxString wildcard = _T("TXT files (*.txt)|*.txt|Data files (*.dat)|*.dat|All files (*.*)|*.*");
	wxString defaultDir = _T("c:\\");
	wxString defaultFilename = wxEmptyString;
	wxString path = wxEmptyString;
	int filterIndex;
	wxTextFile file;
	
	wxFileDialog dialog(this,caption,defaultDir,defaultFilename,wildcard,wxOPEN);
	if(dialog.ShowModal() == wxID_OK){
		path = dialog.GetPath();
		filterIndex = dialog.GetFilterIndex();
		
		//read the file content and put into display
		if(file.Open(path))
		{
			int i, m;
			int dimen_Cap = extractParameter(file[0]);
			int dimen_rho = extractParameter(file[1]);
			int number_J = extractParameter(file[2]);
			int number_K = extractParameter(file[3]);
			int number_M = extractParameter(file[4]);
			int number_C = extractParameter(file[5]);
			m_panel->m_CapDimension->SetValue(dimen_Cap);
			m_panel->ChangeCapDimensionFunction();
			m_panel->m_rhoDimension->SetValue(dimen_rho);
            m_panel->ChangerhoDimensionFunction();
            m_panel->ChangePOVMDimensionFunction();
			m_panel->m_CTEXT->SetValue(number_C);
			m_panel->m_MTEXT->SetValue(number_M);
			m_panel->ChangeCapNumberFunction();			
			m_panel->m_JTEXT->SetValue(number_J);
			m_panel->ChangerhoNumberFunction();
			m_panel->m_KTEXT->SetValue(number_K);
			m_panel->ChangePOVMNumberFunction();
			for(m=0;m<number_C;m++) {
			for(i=0;i<number_M;i++){
				extractCapElement(file[i+6],m+1,i+1,dimen_Cap);
			    }
            }	
			m_panel->m_book->SetPageText(0,path.AfterLast('\\'));
		}
	}
}

void myFrame::OnQuit(wxCommandEvent& e)
{
	Close(TRUE);
}

void myFrame::OnAbout(wxCommandEvent& e)
{
    (void)wxMessageBox(_T("Search for classical Capacity of Quantum Channels(SeCQC) Version 1.0, a program that can calculate the maximum accessible information and the corresponding optimal statistical operators as well as POVMs given a certain quantum channel.\nCopyright (C) 2010 J.W. Shang, K.L. Lee and B.-G. Englert."),
            _T("About SeCQC"), wxOK|wxICON_INFORMATION);
}

//-----------------------------------------------------------
//	Event Handlers for myPanel
//-----------------------------------------------------------

//-----------------------------------------------------------
//	Event Handlers to manipulate CapGrid
//-----------------------------------------------------------
//change the number of Cap_m
void myPanel::ChangeCapNumber(wxSpinEvent& se)
{
	ChangeCapNumberFunction();
}

void myPanel::ChangeCapNumberFunction()
{
    m_InputCapC->SetRange(1,m_CTEXT->GetValue());
    m_InputCapM->SetRange(1,m_MTEXT->GetValue());
	m_information->UpdateCapDimension(m_CTEXT->GetValue(), m_MTEXT->GetValue(),m_CapDimension->GetValue());
}

//modify the dimension of each Cap_m, update the Cap_m matrix displayed
void myPanel::ChangeCapDimension(wxSpinEvent& se)
{	
	ChangeCapDimensionFunction();
}

void myPanel::ChangeCapDimensionFunction()
{
	m_CapDimensionText->SetLabel(wxString::Format(_T("%d"),m_CapDimension->GetValue()));
	m_information->UpdateCapDimension(m_CTEXT->GetValue(),m_MTEXT->GetValue(),m_CapDimension->GetValue());
	CapGrid->setDimension(m_CapDimension->GetValue());
	
	int i, j;
	j = m_InputCapM->GetValue();
	i = m_InputCapC->GetValue();		
	CapGrid->setGrid(m_information->getCap(i-1,j-1),m_CapDimension->GetValue(),m_CapDimension->GetValue(),m_CapRadio->GetSelection());	
}

//display the Cap_m matrix selected by the user
void myPanel::UpdateCap(wxSpinEvent& e)
{
	int i, j;
	j = m_InputCapM->GetValue();
	i = m_InputCapC->GetValue();
	CapGrid->setGrid(m_information->getCap(i-1,j-1),m_CapDimension->GetValue(),m_CapDimension->GetValue(),m_CapRadio->GetSelection());
}

//response to values entered by updating the corresponding Cap_m
void myPanel::OnCapCellValueChanged(wxGridEvent& ev)
{
	int row = ev.GetRow(),
        col = ev.GetCol();
        double d = convertString(CapGrid->GetCellValue(row,col)); 
        m_information->setCapElement(m_InputCapC->GetValue()-1, m_InputCapM->GetValue()-1, row, col, d, m_CapRadio->GetSelection()); 
}
//-----------------------------------------------------------
//	End of event handlers to manipulate CapGrid
//-----------------------------------------------------------

//-----------------------------------------------------------
//	Event Handlers to manipulate rhoGrid and povmGrid
//-----------------------------------------------------------
//change the number of rho_j
void myPanel::ChangerhoNumber(wxSpinEvent& se)
{
    ChangerhoNumberFunction();
}

//change the number of pi_k
void myPanel::ChangePOVMNumber(wxSpinEvent& se)
{
    ChangePOVMNumberFunction();
}

void myPanel::ChangerhoNumberFunction()
{
    m_Displayrho->SetRange(1,m_JTEXT->GetValue());
    m_information->UpdaterhoDimension(m_JTEXT->GetValue(),m_rhoDimension->GetValue());
}

void myPanel::ChangePOVMNumberFunction()
{
    m_DisplayPOVM->SetRange(1,m_KTEXT->GetValue());
    m_information->UpdatePOVMDimension(m_KTEXT->GetValue(),m_rhoDimension->GetValue());
}

//modify the dimension of each rho_j
void myPanel::ChangerhoDimension(wxSpinEvent& se)
{	
	ChangerhoDimensionFunction();
}

//modify the dimension of each pi_k
void myPanel::ChangePOVMDimension(wxSpinEvent& se)
{	
	ChangePOVMDimensionFunction();
}

void myPanel::ChangerhoDimensionFunction()
{
    m_rhoDimensionText->SetLabel(wxString::Format(_T("%d"),m_rhoDimension->GetValue()));
    m_information->UpdaterhoDimension(m_JTEXT->GetValue(),m_rhoDimension->GetValue());
    rhoGrid->setDimension(m_rhoDimension->GetValue());
    
    int i;
    i=m_Displayrho->GetValue();
    rhoGrid->setGrid(m_information->getrho(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_rhoRadio->GetSelection());
}

void myPanel::ChangePOVMDimensionFunction()
{
    m_information->UpdatePOVMDimension(m_KTEXT->GetValue(),m_rhoDimension->GetValue());
    povmGrid->setDimension(m_rhoDimension->GetValue());
    
    int i;
    i=m_Displayrho->GetValue();
    povmGrid->setGrid(m_information->getPOVM(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_povmRadio->GetSelection());
}

//update new rho_j
void myPanel::Updaterho(wxSpinEvent& e)
{
	int i;
	i = m_Displayrho->GetValue();
	rhoGrid->setGrid(m_information->getrho(i-1),m_information->rhoDimension,m_information->rhoDimension,m_rhoRadio->GetSelection());
}

//update new pi_k
void myPanel::UpdatePOVM(wxSpinEvent& e)
{
	int i;
	i = m_DisplayPOVM->GetValue();
	povmGrid->setGrid(m_information->getPOVM(i-1),m_information->povmDimension,m_information->povmDimension,m_povmRadio->GetSelection());
}

//response to values entered by updating the corresponding rho_j
void myPanel::OnrhoCellValueChanged(wxGridEvent& ev)
{
    int row = ev.GetRow(),
        col = ev.GetCol();
    
    double d = convertString(rhoGrid->GetCellValue(row,col));
    m_information->setrhoElement(m_Displayrho->GetValue()-1,row,col,d,m_rhoRadio->GetSelection());
}

//response to values entered by updating the corresponding pi_k
void myPanel::OnPOVMCellValueChanged(wxGridEvent& ev)
{
    int row = ev.GetRow(),
        col = ev.GetCol();
    
    double d = convertString(povmGrid->GetCellValue(row,col));
    m_information->setPOVMElement(m_DisplayPOVM->GetValue()-1,row,col,d,m_povmRadio->GetSelection());
}
//-----------------------------------------------------------
//	End of event handlers to manipulate rhoGrid and povmGrid
//-----------------------------------------------------------

void myPanel::OnCalc(wxCommandEvent& e)
{	    
	this->Enable(false);

    m_information->UpdaterhoDimension(m_JTEXT->GetValue(),m_rhoDimension->GetValue());    	  
	m_information->UpdatePOVMDimension(m_KTEXT->GetValue(),m_rhoDimension->GetValue());	
	m_tol_MI->GetValue().ToDouble(&(m_information->tol_MI));
	m_information->output = m_directory->GetValue() + "\\" + m_output->GetValue(); //output file directory + name	
	m_information->directGradChance = m_directGradChance->GetValue();
	m_information->calcMutualInformation();

    m_Displayrho->SetRange(1,m_information->getJ());
    rhoGrid->setDimension(m_rhoDimension->GetValue());
	m_DisplayPOVM->SetRange(1,m_information->getK());
	povmGrid->setDimension(m_rhoDimension->GetValue());
	int j = m_Displayrho->GetValue(); //to display the optimal rhos
	rhoGrid->setGrid(m_information->getrho(j-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_rhoRadio->GetSelection());
	int i = m_DisplayPOVM->GetValue(); //to display the optimal POVMs
	povmGrid->setGrid(m_information->getPOVM(i-1),m_rhoDimension->GetValue(),m_rhoDimension->GetValue(),m_povmRadio->GetSelection());
	
	wxString s = "";
	s += wxString::Format(_T("%f"),m_information->MI);
	m_displayMI->SetValue(s);
	this->Enable(true);
}

//-----------------------------------------------------------
//	change the output file directory
//-----------------------------------------------------------
void myPanel::OnSave(wxCommandEvent& e)
{
	//open a file dialog to export file
	wxString caption = _T("Choose a directory to save the output file");
	wxString defaultDir = _T("C:\\");
	wxString path;
	
	wxDirDialog dialog(this, caption, defaultDir, wxDD_NEW_DIR_BUTTON);
	if(dialog.ShowModal() == wxID_OK){
        path = dialog.GetPath();
	}
    m_directory->SetValue(path); //set the new output file path
}

//-----------------------------------------------------------
//	reset all values to default
//-----------------------------------------------------------
void myPanel::OnReset(wxCommandEvent& e)
{
    m_CapDimension->SetValue(2);
	m_rhoDimension->SetValue(2);
	m_JTEXT->SetValue(2);
	m_KTEXT->SetValue(5);
	m_MTEXT->SetValue(2);
	m_CTEXT->SetValue(1);
	m_directGradChance->SetValue(0);
	m_tol_MI->SetValue("0.000001");
	m_output->SetValue("output.txt");
	m_directory->SetValue("C:\\");
	m_displayMI->SetValue("");
}
